<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

class Voting extends ObjectModel
{
    public $id_voting;
    public $view = 1;
    public $modal = '';
    public $multiple = 0;
    public $can_close = 1;
    public $can_dismiss = 1;
    public $delay = 30;
    public $on_pc = 1;
    public $on_mobile = 1;
    public $on_tablet = 1;
    public $page_index = 1;
    public $page_category = 1;
    public $page_product = 1;
    public $page_cms = 1;
    public $page_cart = 0;
    public $page_order = 0;
    public $page_orderconfirmation = 1;
    public $page_myaccount = 1;
    public $page_contact = 1;
    public $page_manufacturer = 1;
    public $page_newproducts = 1;
    public $page_pricesdrop = 1;
    public $page_bestsales = 1;
    public $show_title = 0;
    public $active = 0;
    public $date_add;
    public $date_upd;
    public $position;
    public $name;
    public $description;
    public $thanks_header;
    public $thanks_text;

    public $groupBox;
    public $products;
    public $categories;

    public static $definition = [
        'table' => 'voting',
        'primary' => 'id_voting',
        'multilang' => true,
        'fields' => [
            'view' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'modal' => ['type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'dbNullable' => false, 'size' => 16],
            'multiple' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'can_close' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'can_dismiss' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'delay' => ['type' => self::TYPE_INT, 'validate' => 'isInt', 'dbDefault' => '0', 'dbType' => 'int(10)', 'lang' => false],
            'on_pc' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'on_mobile' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'on_tablet' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_index' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_category' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_product' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_cms' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_cart' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_order' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_orderconfirmation' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_myaccount' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_contact' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_manufacturer' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_newproducts' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_pricesdrop' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'page_bestsales' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'show_title' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'active' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'dbNullable' => false],
            'date_upd' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'dbNullable' => false],
            'position' => ['type' => self::TYPE_INT, 'dbDefault' => '0'],
            'name' => ['type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'required' => true, 'size' => 128],
            'description' => ['type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml', 'required' => true, 'size' => 1024],
            'thanks_header' => ['type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'required' => true, 'size' => 128],
            'thanks_text' => ['type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'required' => true, 'size' => 256],
        ],
        'associations' => [
            'shop' => ['type' => self::HAS_MANY, 'field' => 'id_shop', 'object' => 'Shop', 'association' => 'voting_shop'],
            'group' => ['type' => self::HAS_MANY, 'field' => 'id_group', 'object' => 'Group', 'association' => 'voting_group'],
            'category' => ['type' => self::HAS_MANY, 'field' => 'id_category', 'object' => 'Category', 'association' => 'voting_category'],
            'product' => ['type' => self::HAS_MANY, 'field' => 'id_product', 'object' => 'Product', 'association' => 'voting_product'],
            'option' => ['type' => self::HAS_MANY, 'field' => 'id_voting_option', 'object' => 'VotingOption', 'association' => 'voting_option'],
            'vote' => ['type' => self::HAS_MANY, 'field' => 'id_voting_vote', 'object' => 'VotingVote', 'association' => 'voting_vote']
        ]
    ];

    public static function getVotingThankYou($id_voting, $idLang = null)
    {
        if (!$idLang) {
            $idLang = (int) Configuration::get('PS_LANG_DEFAULT');
        }
        return Db::getInstance()->getRow(
		    'SELECT `thanks_header` `header`, `thanks_text` `text` FROM `' . _DB_PREFIX_ . 'voting_lang`
		    WHERE `id_voting` = ' . (int) $id_voting . ' AND `id_lang` = ' . (int) $idLang
        );
    }

    public static function canBeClosed($id_voting)
    {
        return (int) Db::getInstance()->getValue(
            'SELECT `can_close`  FROM `' . _DB_PREFIX_ . 'voting` WHERE `id_voting` = ' . (int) $id_voting
        );
    }

    public static function canBeDismissed($id_voting)
    {
        return (int) Db::getInstance()->getValue(
            'SELECT `can_dismiss`  FROM `' . _DB_PREFIX_ . 'voting` WHERE `id_voting` = ' . (int) $id_voting
        );
    }

    public static function getVotingSimple($id_voting)
    {
        return Db::getInstance()->getRow(
            'SELECT * FROM `' . _DB_PREFIX_ . 'voting` WHERE `id_voting` = ' . (int) $id_voting
        );
    }

    public static function getVoting($id_voting, $idLang = null)
    {
        if (!$idLang) {
            $idLang = (int) Configuration::get('PS_LANG_DEFAULT');
        }
        return Db::getInstance()->getRow(
		    'SELECT v.*, vl.* FROM `' . _DB_PREFIX_ . 'voting` v
		    LEFT JOIN `' . _DB_PREFIX_ . 'voting_lang` vl ON (v.`id_voting` = vl.`id_voting` AND vl.`id_lang` = ' . (int) $idLang . ')
            WHERE v.`id_voting` = ' . (int) $id_voting
        );
    }

    public static function getVotingWithOptions($id_voting, $idLang = null)
    {
        if (!$idLang) {
            $idLang = (int) Configuration::get('PS_LANG_DEFAULT');
        }
        if ($voting = self::getVoting($id_voting, $idLang)) {
            $voting['options'] = Db::getInstance()->executeS(
                'SELECT vo.*, vol.* FROM `' . _DB_PREFIX_ . 'voting_option` vo
		        LEFT JOIN `' . _DB_PREFIX_ . 'voting_option_lang` vol
                ON vol.`id_voting_option` = vo.`id_voting_option` AND vol.`id_lang` = ' . (int) $idLang . '
                WHERE vo.`id_voting` = ' . (int) $id_voting
            );
            return $voting;
        }
    }

    public static function getVotings($idLang, $withShop = true)
    {
        return Db::getInstance()->executeS(
		    'SELECT DISTINCT v.id_voting, v.*, vl.* FROM `' . _DB_PREFIX_ . 'voting` v
		    ' . ($withShop ? Shop::addSqlAssociation('voting', 'v') : '') . '
		    LEFT JOIN `' . _DB_PREFIX_ . 'voting_lang` vl ON (v.`id_voting` = vl.`id_voting` AND vl.`id_lang` = ' . (int) $idLang . ')
		    ORDER BY v.`position` ASC'
        );
    }

    public function getGroups()
    {
        $cacheId = $this->getGroupsCacheId();
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->select('vg.`id_group`');
            $sql->from('voting_group', 'vg');
            $sql->where('vg.`id_voting` = ' . (int) $this->id);
            $result = Db::getInstance()->executeS($sql) ?? [];
            $groups = [];
            foreach ($result as $group) {
                $groups[] = (int) $group['id_group'];
            }
            Cache::store($cacheId, $groups);
            return $groups;
        }
        return Cache::retrieve($cacheId);
    }

    public function getShops()
    {
        $cacheId = $this->getShopsCacheId();
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->select('vg.`id_shop`');
            $sql->from('voting_shop', 'vg');
            $sql->where('vg.`id_voting` = ' . (int) $this->id);
            $result = Db::getInstance()->executeS($sql) ?? [];
            $shops = [];
            foreach ($result as $shop) {
                $shops[] = (int) $shop['id_shop'];
            }
            Cache::store($cacheId, $shops);
            return $shops;
        }
        return Cache::retrieve($cacheId);
    }

    public function getProducts()
    {
        $cacheId = $this->getProductsCacheId();
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->select('vg.`id_product`');
            $sql->from('voting_product', 'vg');
            $sql->where('vg.`id_voting` = ' . (int) $this->id);
            $result = Db::getInstance()->executeS($sql) ?? [];
            $products = [];
            foreach ($result as $product) {
                $products[] = (int) $product['id_product'];
            }
            Cache::store($cacheId, $products);
            return $products;
        }
        return Cache::retrieve($cacheId);
    }

    public function getCategories()
    {
        $cacheId = $this->getCategoriesCacheId();
        if (!Cache::isStored($cacheId)) {
            $sql = new DbQuery();
            $sql->select('vg.`id_category`');
            $sql->from('voting_category', 'vg');
            $sql->where('vg.`id_voting` = ' . (int) $this->id);
            $result = Db::getInstance()->executeS($sql) ?? [];
            $categories = [];
            foreach ($result as $category) {
                $categories[] = (int) $category['id_category'];
            }
            Cache::store($cacheId, $categories);
            return $categories;
        }
        return Cache::retrieve($cacheId);
    }

    public function add($autoDate = true, $nullValues = false)
    {
        if ($this->position <= 0) {
            $this->position = self::getHigherPosition() + 1;
        }
        $ret = parent::add($autoDate, $nullValues);
        if ($ret) {
            $this->updateGroups(($this->groupBox !== null) ? $this->groupBox : []);
            $this->updateProducts($this->products);
            $this->updateCategories($this->categories);
        }
        return $ret;
    }

    public function update($nullValues = false)
    {
        $ret = parent::update($nullValues);
        if ($ret) {
            $this->updateGroups($this->groupBox);
            $this->updateProducts($this->products);
            $this->updateCategories($this->categories);
        }
        return $ret;
    }

    public function deleteSelection($selection)
    {
        foreach ($selection as $value) {
            $obj = new Voting($value);
            if (!$obj->delete()) {
                return false;
            }
        }
        return true;
    }

    public function delete()
    {
        $ret = parent::delete();
        if ($ret) {
            $this->cleanCategories();
            $this->cleanGroups();
            $this->cleanProducts();
            $this->cleanShops();
            VotingVote::deleteByVoting((int) $this->id);
            VotingOption::deleteByVoting((int) $this->id);
            self::cleanPositions();
        }
        return $ret;
    }

    public function updateCategories($list)
    {
        if (!empty($list)) {
            $this->cleanCategories();
            $this->addCategories($list);
        }
    }

    public function updateGroups($list)
    {
        if (!empty($list)) {
            $this->cleanGroups();
            $this->addGroups($list);
        }
    }

    public function updateProducts($list)
    {
        if (!empty($list)) {
            $this->cleanProducts();
            $this->addProducts($list);
        }
    }

    public function updateShops($list)
    {
        if (!empty($list)) {
            $this->cleanShops();
            $this->addShops($list);
        }
    }

    public function cleanCategories()
    {
        $result = Db::getInstance()->delete('voting_category', 'id_voting = ' . (int) $this->id);
        Cache::clean($this->getCategoriesCacheId());
        return $result;
    }

    public function cleanGroups()
    {
        $result = Db::getInstance()->delete('voting_group', 'id_voting = ' . (int) $this->id);
        Cache::clean($this->getGroupsCacheId());
        return $result;
    }

    public function cleanProducts()
    {
        $result = Db::getInstance()->delete('voting_product', 'id_voting = ' . (int) $this->id);
        Cache::clean($this->getProductsCacheId());
        return $result;
    }

    public function cleanShops()
    {
        $result = Db::getInstance()->delete('voting_shop', 'id_voting = ' . (int) $this->id);
        Cache::clean($this->getShopsCacheId());
        return $result;
    }

    public function addCategories($categories)
    {
        foreach ($categories as $category) {
            if ($category !== false) {
                Db::getInstance()->insert('voting_category', ['id_voting' => (int) $this->id, 'id_category' => (int) $category]);
            }
        }
        Cache::clean($this->getCategoriesCacheId());
    }

    public function addGroups($groups)
    {
        foreach ($groups as $group) {
            if ($group !== false) {
                Db::getInstance()->insert('voting_group', ['id_voting' => (int) $this->id, 'id_group' => (int) $group]);
            }
        }
        Cache::clean($this->getGroupsCacheId());
    }

    public function addProducts($products)
    {
        foreach ($products as $product) {
            if ($product !== false) {
                Db::getInstance()->insert('voting_product', ['id_voting' => (int) $this->id, 'id_product' => (int) $product]);
            }
        }
        Cache::clean($this->getProductsCacheId());
    }

    public function addShops($shops)
    {
        foreach ($shops as $shop) {
            if ($shop !== false) {
                Db::getInstance()->insert('voting_shop', ['id_voting' => (int) $this->id, 'id_shop' => (int) $shop]);
            }
        }
        Cache::clean($this->getShopsCacheId());
    }

    public function updatePosition($way, $position, $id = null)
    {
        if (!$res = Db::getInstance()->executeS(
            'SELECT `position`, `id_voting`
			FROM `' . _DB_PREFIX_ . 'voting`
			WHERE `id_voting` = ' . (int) ($id ? $id : $this->id) . '
			ORDER BY `position` ASC'
        )) {
            return false;
        }
        foreach ($res as $r) {
            if ((int) $r['id_voting'] == (int) $this->id) {
                $moved = $r;
            }
        }
        if (!isset($moved) || !isset($position)) {
            return false;
        }
        return Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'voting`
            SET `position`= `position` ' . ($way ? '- 1' : '+ 1') . '
            WHERE `position`
            ' . ($way
                ? '> ' . (int) $moved['position'] . ' AND `position` <= ' . (int) $position
                : '< ' . (int) $moved['position'] . ' AND `position` >= ' . (int) $position))
        && Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'voting`
            SET `position` = ' . (int) $position . '
            WHERE `id_voting`=' . (int) $moved['id_voting']);
    }

    public static function cleanPositions()
    {
        Db::getInstance()->execute('SET @i = -1', false);
        $sql = 'UPDATE `' . _DB_PREFIX_ . 'voting` SET `position` = @i:=@i+1 ORDER BY `position` ASC';
        return (bool) Db::getInstance()->execute($sql);
    }

    public static function getHigherPosition()
    {
        $sql = 'SELECT MAX(`position`) FROM `' . _DB_PREFIX_ . 'voting`';
        $position = Db::getInstance()->getValue($sql);
        return (is_numeric($position)) ? $position : -1;
    }

    private function getGroupsCacheId(): string
    {
        return 'Voting::getGroups_' . (int) $this->id;
    }

    private function getShopsCacheId(): string
    {
        return 'Voting::getShops_' . (int) $this->id;
    }

    private function getProductsCacheId(): string
    {
        return 'Voting::getProducts_' . (int) $this->id;
    }

    private function getCategoriesCacheId(): string
    {
        return 'Voting::getCategories_' . (int) $this->id;
    }

}