<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

class VotingOption extends ObjectModel
{
    public $id_voting_option;
    public $id_voting;
    public $show_title = 0;
    public $show_count = 0;
    public $active = 1;
    public $date_add;
    public $date_upd;
    public $position;
    public $name;
    public $description;
    public $button;
    public static $definition = [
        'table' => 'voting_option',
        'primary' => 'id_voting_option',
        'multilang' => true,
        'fields' => [
            'id_voting' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'show_title' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'show_count' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'active' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true, 'dbDefault' => '0'],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'dbNullable' => false],
            'date_upd' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'dbNullable' => false],
            'position' => ['type' => self::TYPE_INT, 'dbDefault' => '0'],
            'name' => ['type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'required' => true, 'size' => 128],
            'description' => ['type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml', 'required' => true, 'size' => 1024],
            'button' => ['type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'required' => false, 'size' => 128]
        ],
        'associations' => [
            'voting' => ['type' => self::HAS_ONE, 'field' => 'id_voting', 'object' => 'Voting'],
            'vote' => ['type' => self::HAS_MANY, 'field' => 'id_voting_vote', 'object' => 'VotingVote', 'association' => 'voting_vote']
        ]
    ];

    public function add($autoDate = true, $nullValues = false)
    {
        if ($this->position <= 0) {
            $this->position = self::getHigherPosition((int) $this->id_voting) + 1;
        }
        return parent::add($autoDate, $nullValues);;
    }

    public function delete()
    {
        $ret = parent::delete();
        if ($ret) {
            self::cleanPositions($this->id_voting);
            VotingVote::deleteByVotingOption((int) $this->id);
        }
        return $ret;
    }

    public static function deleteByVoting($id_voting)
    {
        $sql = 'SELECT `id_voting_option` FROM `' . _DB_PREFIX_ . 'voting_option` WHERE `id_voting` = ' . (int) $id_voting;
        if ($res = Db::getInstance()->executeS($sql)) {
            foreach ($res as $r) {
                $obj = new VotingOption((int) $r['id_voting_option']);
                $obj->delete();
            }
        }
    }

    public function updatePosition($way, $position, $id = null)
    {
        if (!$res = Db::getInstance()->executeS(
            'SELECT `position`, `id_voting_option`
			FROM `' . _DB_PREFIX_ . 'voting_option`
			WHERE `id_voting_option` = ' . (int) ($id ? $id : $this->id) . ' AND `id_voting` = ' . $this->id_voting . '
			ORDER BY `position` ASC'
        )) {
            return false;
        }
        foreach ($res as $r) {
            if ((int) $r['id_voting_option'] == (int) $this->id) {
                $moved = $r;
            }
        }
        if (!isset($moved) || !isset($position)) {
            return false;
        }
        return Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'voting_option`
            SET `position`= `position` ' . ($way ? '- 1' : '+ 1') . '
            WHERE `id_voting` = ' . $this->id_voting . ' AND `position`
            ' . ($way
                ? '> ' . (int) $moved['position'] . ' AND `position` <= ' . (int) $position
                : '< ' . (int) $moved['position'] . ' AND `position` >= ' . (int) $position))
        && Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'voting_option`
            SET `position` = ' . (int) $position . '
            WHERE `id_voting` = ' . $this->id_voting . ' AND `id_voting_option`=' . (int) $moved['id_voting_option']);
    }

    public static function cleanPositions($id_voting)
    {
        Db::getInstance()->execute('SET @i = -1', false);
        $sql = 'UPDATE `' . _DB_PREFIX_ . 'voting_option` SET `position` = @i:=@i+1
         WHERE `id_voting` = ' . $id_voting . ' ORDER BY `position` ASC';
        return (bool) Db::getInstance()->execute($sql);
    }

    public static function getHigherPosition($id_voting)
    {
        $sql = 'SELECT MAX(`position`) FROM `' . _DB_PREFIX_ . 'voting_option` WHERE `id_voting` = ' . $id_voting;
        $position = Db::getInstance()->getValue($sql);
        return (is_numeric($position)) ? $position : -1;
    }
}