<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

class VotingVote extends ObjectModel
{
    public $id_voting_vote;
    public $id_voting;
    public $dismissed;
    public $id_voting_option;
    public $id_customer;
    public $id_guest;
    public $ip_address;
    public $date_add;
    public static $definition = [
        'table' => 'voting_vote',
        'primary' => 'id_voting_vote',
        'multilang' => false,
        'fields' => [
            'id_voting' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'dismissed' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'dbDefault' => '0'],
            'id_voting_option' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'id_customer' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'id_guest' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId'],
            'ip_address' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'dbNullable' => false]
        ],
        'associations' => [
            'voting' => ['type' => self::HAS_ONE, 'field' => 'id_voting', 'object' => 'Voting'],
            'option' => ['type' => self::HAS_MANY, 'field' => 'id_voting_option', 'object' => 'VotingOption'],
        ]
    ];

    public static function isVoted($id_voting, $id_customer, $id_guest, $ip_address)
    {
        $mode = (int) Configuration::get('SIMPLEVOTINGS_ACTION_MODE');
        if ($mode === 0) {
            if ($id_customer) {
                $sql_add = '`id_customer` = ' . (int) $id_customer;
            } else {
                $sql_add = '`id_guest` = ' . (int) $id_guest;
            }
        } elseif ($mode === 1) {
            $sql_add = '`ip_address` = ' . (int) $ip_address;
        } elseif ($mode === 2) {
            if ($id_customer) {
                $sql_add = '`id_customer` = ' . (int) $id_customer;
            } else {
                $sql_add = '`ip_address` = ' . (int) $ip_address;
            }
        }
        return (int) Db::getInstance()->getValue(
            'SELECT `id_voting_vote` FROM `' . _DB_PREFIX_ . 'voting_vote`
            WHERE `id_voting` = ' . (int) $id_voting . ' AND ' . $sql_add
        );
    }

    public static function deleteByVoting($id_voting)
    {
        return Db::getInstance()->execute(
            'DELETE FROM `' . _DB_PREFIX_ . 'voting_vote` WHERE `id_voting` = ' . (int) $id_voting
        );
    }

    public static function deleteByVotingOption($id_voting_option)
    {
        return Db::getInstance()->execute(
            'DELETE FROM `' . _DB_PREFIX_ . 'voting_vote` WHERE `id_voting_option` = ' . (int) $id_voting_option
        );
    }
}
