<?php
/**<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

require_once(dirname(__FILE__).'/../../classes/Voting.php');
require_once(dirname(__FILE__).'/../../classes/VotingOption.php');
require_once(dirname(__FILE__).'/../../classes/VotingVote.php');

class AdminSimpleVotingsVotingController extends ModuleAdminController
{
    public $bootstrap = true;
    protected $position_identifier = 'id_voting';
    protected $voting_name;

    public $modal_sizes = [
        [
            'id' => '',
            'name' => 'Default'
        ],
        [
            'id' => 'sm',
            'name' => 'Small (sm)'
        ],
        [
            'id' => 'md',
            'name' => 'Medium (md)'
        ],
        [
            'id' => 'lg',
            'name' => 'Large (lg)'
        ],
        [
            'id' => 'xl',
            'name' => 'Very Large (xl)'
        ],
        [
            'id' => 'fs',
            'name' => 'Full screen (fs)'
        ]
    ];

    public function __construct()
    {
        $this->table = 'voting';
        $this->className = 'Voting';
        $this->list_id = 'voting';
        $this->identifier = 'id_voting';
        $this->lang = true;
        $this->list_no_link = true;
        parent::__construct();
        if (!$this->module->active) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminHome'));
        }
        $this->id_lang = (int) $this->context->language->id;
        $this->id_shop = (int) $this->context->shop->id;
        $this->employee = $this->context->employee;
        $this->access = Profile::getProfileAccess($this->employee->id_profile, $this->id);
        $this->ps_groups = Group::getGroups($this->id_lang, $this->id_shop);

        $this->_defaultOrderBy = 'position';
        $this->_defaultOrderWay = 'ASC';

        $this->fields_list = [
            'id_voting' => [
                'title' => 'ID',
                'class' => 'fixed-width-xs',
                'align' => 'center'
            ],
            'name' => [
                'title' => 'Voting Name',
                'align' => 'center',
                'filter_key' => 'b!name'
            ],
            'description' => [
                'title' => 'Voting Description',
                'align' => 'center',
                'filter_key' => 'b!description',
                'callback' => 'fixCellDescription'
            ],
            'options' => [
                'title' => 'Voting Options',
                'align' => 'center',
                'filter_key' => 'ol!name',
                'callback' => 'fixCellOptions'
            ],
            'active' => [
                'title' => 'Active',
                'class' => 'fixed-width-sm',
                'align' => 'center',
                'active' => 'toggleActive',
                'type' => 'bool',
                'orderby' => false,
                'filter_key' => 'a!active',
                'ajax' => true
            ],
            'position' => [
                'title' => 'Position',
                'filter_key' => 'a!position',
                'search' => false,
                'align' => 'center',
                'class' => 'fixed-width-xs',
                'position' => 'position'
            ],
            'date_add' => [
                'title' => 'Creation date',
                'type'=>'datetime',
                'align' => 'center',
                'filter_key'=>'a!date_add'
            ],
            'date_upd' => [
                'title' => 'Update date',
                'align' => 'center',
                'type'=>'datetime',
                'filter_key'=>'a!date_upd'
            ]
        ];
        $this->bulk_actions = [
            'delete' => [
                'text' => 'Delete selected',
                'confirm' => 'Delete selected items?'
            ]
        ];
    }

    public function fixCellOptions($value)
    {
        foreach (explode('<br>', $value) as $n) {
            $fixed_array[] = '<span>' . $n . '</span>';
        }
        return implode('<br>', $fixed_array);
    }

    public function fixCellDescription($value)
    {
        return strip_tags($value);
    }

    public function init()
    {
        parent::init();
    }

    public function renderList()
    {
        $this->addRowAction('view');
        $this->addRowAction('edit');
        $this->addRowAction('delete');
        return parent::renderList();
    }

    protected function setTypeVoting()
    {
        $this->position_identifier = 'id_voting';
        $this->table = 'voting';
        $this->className = 'Voting';
        $this->identifier = 'id_voting';
        $this->_select = ' GROUP_CONCAT(CONCAT(ol.`name`, " (", IFNULL(`cnt`, "0"), ")") separator "<br>") `options` ';
        $this->_join = ' LEFT JOIN `' . _DB_PREFIX_ . 'voting_option` o ON (o.`id_voting` = a.`id_voting`)';
        $this->_join .= ' LEFT JOIN `' . _DB_PREFIX_ . 'voting_option_lang` ol ON (ol.`id_voting_option` = o.`id_voting_option` AND ol.`id_lang` = ' . $this->id_lang . ') ';
        $this->_join .= ' LEFT JOIN (
                            SELECT `id_voting_option`, COUNT(*) AS `cnt` FROM `' . _DB_PREFIX_ . 'voting_vote` WHERE `dismissed` = 0 GROUP BY `id_voting_option`
                          ) cn ON (cn.`id_voting_option` = o.`id_voting_option`) ';
        $this->_group = ' GROUP BY a.`id_voting` ';
    }

    protected function setTypeOption()
    {
        $this->position_identifier = 'id_voting_option';
        $this->table = 'voting_option';
        $this->className = 'VotingOption';
        $this->identifier = 'id_voting_option';
        $this->_select = ' a.`id_voting_option`, vl.`name` `voting`, COUNT(cn.`id_voting_vote`) as `votes` ';
        $this->_join = ' LEFT JOIN `' . _DB_PREFIX_ . 'voting_lang` vl ON (vl.`id_voting` = a.`id_voting` AND vl.`id_lang` = ' . $this->id_lang . ') ';
        $this->_join .= ' LEFT JOIN `' . _DB_PREFIX_ . 'voting_vote` cn ON (cn.`id_voting_option` = a.`id_voting_option` AND cn.`dismissed` = 0) ';
        $this->_group = ' GROUP BY a.`id_voting_option` ';
    }

    public function renderView()
    {
        if ($id = (int) Tools::getValue('id_voting')) {
            $this->setTypeOption();
            $this->list_id = 'voting_option';
            $this->lang = true;
            $this->addRowAction('edit');
            $this->addRowAction('delete');
            if (!Validate::isLoadedObject($obj = new Voting((int) $id))) {
                $this->errors[] = 'An error occurred while updating the status for an object. <b>' . $this->table . '</b> (cannot load object)';
                return;
            }
            $this->voting_name = $obj->name;
            $this->toolbar_title = $this->voting_name[$this->context->language->id];
            $this->fields_list = [
                'id_voting_option' => [
                    'title' => 'ID',
                    'class' => 'fixed-width-xs',
                    'align' => 'center'
                ],
                'voting' => [
                    'title' => 'Voting Name',
                    'align' => 'center',
                    'filter_key' => 'vl!name'
                ],
                'name' => [
                    'title' => 'Option Name',
                    'align' => 'center',
                    'filter_key' => 'b!name'
                ],
                'description' => [
                    'title' => 'Option Description',
                    'align' => 'center',
                    'filter_key' => 'b!description',
                    'callback' => 'fixCellDescription'
                ],
                'votes' => [
                    'title' => 'Votes',
                    'align' => 'center',
                    'filter_key' => 'cn!id_voting_vote'
                ],
                'active' => [
                    'title' => 'Active',
                    'class' => 'fixed-width-sm',
                    'align' => 'center',
                    'active' => 'toggleActive',
                    'type' => 'bool',
                    'orderby' => false,
                    'filter_key' => 'a!active',
                    'ajax' => true
                ],
                'position' => [
                    'title' => 'Position',
                    'filter_key' => 'a!position',
                    'search' => false,
                    'align' => 'center',
                    'class' => 'fixed-width-xs',
                    'position' => 'position'
                ],
                'date_add' => [
                    'title' => 'Creation date',
                    'type'=>'datetime',
                    'align' => 'center',
                    'filter_key'=>'a!date_add'
                ],
                'date_upd' => [
                    'title' => 'Update date',
                    'align' => 'center',
                    'type'=>'datetime',
                    'filter_key'=>'a!date_upd'
                ]
            ];
            $this->bulk_actions = [
                'delete' => [
                    'text' => 'Delete selected',
                    'confirm' => 'Delete selected items?'
                ]
            ];
            $this->_where = sprintf('AND a.`id_voting` = %d', (int) $id);
            self::$currentIndex = self::$currentIndex . '&id_voting=' . (int) $id . '&viewvoting';
            $this->processFilter();
            return parent::renderList();
        }
    }

    public function setMedia($var = false)
    {
        parent::setMedia($var);
        $this->context->controller->addJQueryPlugin('select2');
        $this->context->controller->addJS(_MODULE_DIR_ . $this->module->name . '/views/js/back.js');
    }

    public function renderForm()
    {
        $inputs = [
            [
                'name' => 'name',
                'type' => 'text',
                'label' => 'Title or name of voting',
                'lang' => true,
                'required' => true
            ],
            [
                'name' => 'show_title',
                'type' => 'switch',
                'label' => 'Show or hide title in front',
                'values' => [
                    ['id' => 'show_title_on', 'value' => 1, 'label' => 'Show'],
                    ['id' => 'show_title_off', 'value' => 0, 'label' => 'Hide']
                ]
            ],
            [
                'name' => 'description',
                'type' => 'textarea',
                'label' => 'Question or description of voting (HTML allowed)',
                'lang' => true,
                'required' => true,
                'autoload_rte' => true
            ],
            [
                'name' => 'thanks_header',
                'type' => 'text',
                'label' => 'Thank You message header',
                'lang' => true,
                'required' => true
            ],
            [
                'name' => 'thanks_text',
                'type' => 'text',
                'label' => 'Thank You message text',
                'lang' => true,
                'required' => true
            ],
            [
                'name' => 'delay',
                'type' => 'text',
                'label' => 'Delay in seconds before voting displayed',
                'class' => 'fixed-width-lg',
                'required' => true
            ],
            [
                'name' => 'view',
                'type' => 'switch',
                'label' => 'Display options horizontal or vertical',
                'values' => [
                    ['id' => 'view_on', 'value' => 1, 'label' => 'Horizontal'],
                    ['id' => 'view_off', 'value' => 0, 'label' => 'Vertical']
                ]
            ],
            [
                'name' => 'modal',
                'type' => 'select',
                'lang' => false,
                'multiple' => false,
                'label' => 'Voting modal window size',
                'options' => [
                    'query' => $this->modal_sizes,
                    'id' => 'id',
                    'name' => 'name',
                ],
                'class' => 'fixed-width-xxl'
            ],
            [
                'name' => 'multiple',
                'type' => 'switch',
                'label' => 'Multiple options can be selected or single voted',
                'values' => [
                    ['id' => 'multiple_on', 'value' => 1, 'label' => 'Multiple'],
                    ['id' => 'multiple_off', 'value' => 0, 'label' => 'Single']
                ]
            ],
            [
                'name' => 'can_close',
                'type' => 'switch',
                'label' => 'Allow to customer close voting',
                'values' => [
                    ['id' => 'close_on', 'value' => 1, 'label' => 'Allow'],
                    ['id' => 'close_off', 'value' => 0, 'label' => 'Reject']
                ]
            ],
            [
                'name' => 'can_dismiss',
                'type' => 'switch',
                'label' => 'Allow to customer dismiss voting',
                'values' => [
                    ['id' => 'dismiss_on', 'value' => 1, 'label' => 'Allow'],
                    ['id' => 'dismiss_off', 'value' => 0, 'label' => 'Reject']
                ]
            ],
            [
                'name' => 'on_pc',
                'type' => 'switch',
                'label' => 'Display on PC',
                'values' => [
                    ['id' => 'on_pc_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => 'on_pc_off', 'value' => 0, 'label' => 'No']
                ]
            ],
            [
                'name' => 'on_mobile',
                'type' => 'switch',
                'label' => 'Display on mobile',
                'values' => [
                    ['id' => 'on_mobile_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => 'on_mobile_off', 'value' => 0, 'label' => 'No']
                ]
            ],
            [
                'name' => 'on_tablet',
                'type' => 'switch',
                'label' => 'Display on tablet',
                'values' => [
                    ['id' => 'on_tablet_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => 'on_tablet_off', 'value' => 0, 'label' => 'No']
                ]
            ],
            [
                'type' => 'group',
                'label' => 'Voting access groups',
                'name' => 'groupBox',
                'values' => $this->ps_groups,
                'required' => false,
                'col' => '6',
                'hint' => 'Select one or more groups.',
                'desc' => 'Visitor, Guest, Customer groups will be added if no one group selected.'
            ],
            [
                'name' => 'active',
                'type' => 'switch',
                'label' => 'Activate voting',
                'values' => [
                    ['id' => 'active_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => 'active_off', 'value' => 0, 'label' => 'No']
                ],
                'desc' => '<span class="text-danger">Make sure you added voting OPTIONS before activate.</span>',
            ]
        ];
        foreach ($this->module->page_names as $k => $v) {
            $inputs[] = [
                'name' => 'page_' . $k,
                'type' => 'switch',
                'label' => 'Display on ' . $v . ' page',
                'values' => [
                    ['id' => $k . '_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => $k . '_off', 'value' => 0, 'label' => 'No']
                ]
            ];
            if ($k == 'product') {
                $inputs[] = [
                    'name' => 'products',
                    'type' => 'select',
                    'lang' => false,
                    'multiple' => true,
                    'label' => 'Display on specific products only',
                    'options' => [
                        'query' => Product::getProducts($this->id_lang, 0, 0, 'name', 'ASC', false, true),
                        'id' => 'id_product',
                        'name' => 'name',
                    ],
                    'class' => 'fixed-width-xxl select2'
                ];
            }
            if ($k == 'category') {
                $inputs[] = [
                    'name' => 'categories',
                    'type' => 'select',
                    'multiple' => true ,
                    'label' => 'Display on specific categories only',
                    'options' => [
                        'query' => Category::getCategories($this->id_lang, true, false, 'AND c.`id_category` > 1', 'ORDER BY cl.`name` ASC'),
                        'id' => 'id_category',
                        'name' => 'name',
                    ],
                    'class' => 'fixed-width-xxl select2'
                ];
            }
        }
        if (!($obj = $this->loadObject(true))) {
            return;
        }
        $this->toolbar_title = 'Add a new voting';
        $this->fields_form = [
            'legend' => [
                'title' => isset($obj->name[$this->id_lang]) ? 'Edit voting' . $obj->name[$this->id_lang] : 'Add new voting',
                'icon' => 'icon-list-ul'
            ],
            'input' => $inputs,
            'submit' => [
                'title' => 'Save',
            ]
        ];
        $voting_groups = $obj->getGroups();
        if (empty($voting_groups)) {
            $preselected = [Configuration::get('PS_UNIDENTIFIED_GROUP'), Configuration::get('PS_GUEST_GROUP'), Configuration::get('PS_CUSTOMER_GROUP')];
            $voting_groups = array_merge($voting_groups, $preselected);
        }
        foreach ($this->ps_groups as $group) {
            $this->fields_value['groupBox_' . $group['id_group']] = Tools::getValue('groupBox_' . $group['id_group'], (in_array($group['id_group'], $voting_groups)));
        }
        $this->fields_value['products[]'] = $obj->getProducts() ?? [];
        $this->fields_value['categories[]'] = $obj->getCategories() ?? [];
        if (Shop::isFeatureActive()) {
            $this->fields_form['input'][] = [
                'type' => 'shop',
                'label' => 'Store association',
                'name' => 'checkBoxShopAsso',
            ];
        }
        return parent::renderForm();
    }

    public function initPageHeaderToolbar()
    {
        if (empty($this->display)) {
            $this->page_header_toolbar_btn['new_voting'] = [
                'href' => self::$currentIndex . '&addvoting&token=' . $this->token,
                'desc' => 'Add new voting',
                'icon' => 'process-icon-new',
            ];
            $this->page_header_toolbar_btn['new_voting_option'] = [
                'href' => self::$currentIndex . '&addvoting_option&id_voting=' . (int) Tools::getValue('id_voting') . '&token=' . $this->token,
                'desc' => 'Add new voting option',
                'icon' => 'process-icon-new',
            ];
        }
        if ($this->display == 'view') {
            $this->page_header_toolbar_btn['new_voting_option'] = [
                'href' => self::$currentIndex . '&addvoting_option&id_voting=' . (int) Tools::getValue('id_voting') . '&token=' . $this->token,
                'desc' => 'Add new voting option',
                'icon' => 'process-icon-new',
            ];
        }
        parent::initPageHeaderToolbar();
    }

    public function initToolbar()
    {
        switch ($this->display) {
            case 'editVotingOption':
            case 'add':
            case 'edit':
                $this->toolbar_btn['save'] = [
                    'href' => '#',
                    'desc' => 'Save'
                ];
                if ($this->display == 'editVotingOption') {
                    $this->toolbar_btn['save-and-stay'] = [
                        'short' => 'SaveAndStay',
                        'href' => '#',
                        'desc' => 'Save and add another option',
                        'force_desc' => true
                    ];
                }
                $back = Tools::safeOutput(Tools::getValue('back', ''));
                if (empty($back)) {
                    $back = self::$currentIndex . '&token=' . $this->token;
                }
                $this->toolbar_btn['back'] = [
                    'href' => $back,
                    'desc' => 'Back to the list'
                ];
                break;
            case 'view':
                $this->toolbar_btn['newAttributes'] = [
                    'href' => self::$currentIndex . '&addvoting_option&id_voting=' . (int) Tools::getValue('id_voting') . '&token=' . $this->token,
                    'desc' => 'Add new option'
                ];
                $this->toolbar_btn['back'] = [
                    'href' => self::$currentIndex . '&token=' . $this->token,
                    'desc' => 'Back to the list'
                ];
                break;
            default:
                parent::initToolbar();
        }
    }

    public function initFormVotingOption()
    {
        $this->setTypeOption();
        $inputs = [
            [
                'type' => 'select',
                'label' => 'Voting',
                'name' => 'id_voting',
                'options' => [
                    'query' => Voting::getVotings($this->id_lang),
                    'id' => 'id_voting',
                    'name' => 'name',
                ],
                'required' => true,
            ],
            [
                'name' => 'name',
                'type' => 'text',
                'label' => 'Title or name option',
                'lang' => true,
                'required' => true
            ],
            [
                'name' => 'show_title',
                'type' => 'switch',
                'label' => 'Show or hide title in front',
                'values' => [
                    ['id' => 'show_title_on', 'value' => 1, 'label' => 'Show'],
                    ['id' => 'show_title_off', 'value' => 0, 'label' => 'Hide']
                ]
            ],
            [
                'name' => 'show_count',
                'type' => 'switch',
                'label' => 'Show or hide count of votes in front',
                'values' => [
                    ['id' => 'show_count_on', 'value' => 1, 'label' => 'Show'],
                    ['id' => 'show_count_off', 'value' => 0, 'label' => 'Hide']
                ]
            ],
            [
                'name' => 'description',
                'type' => 'textarea',
                'label' => 'Option description (HTML allowed)',
                'lang' => true,
                'required' => true,
                'autoload_rte' => true
            ],
            [
                'name' => 'button',
                'type' => 'text',
                'label' => 'Action button or checkbox text (example: Vote!)',
                'lang' => true,
                'required' => false
            ],
            [
                'name' => 'active',
                'type' => 'switch',
                'label' => 'Activate option',
                'values' => [
                    ['id' => 'active_on', 'value' => 1, 'label' => 'Yes'],
                    ['id' => 'active_off', 'value' => 0, 'label' => 'No']
                ]
            ]
        ];
        $this->fields_form[0]['form'] = [
            'legend' => [
                'title' => 'Voting option',
                'icon' => 'icon-list-ul'
            ],
            'input' => $inputs,
            'submit' => [
                'title' => 'Save',
            ],
            'buttons' => [
                'save-and-stay' => [
                    'title' => 'Save then add another option',
                    'name' => 'submitAdd' . $this->table . 'AndStay',
                    'type' => 'submit',
                    'class' => 'btn btn-default pull-right',
                    'icon' => 'process-icon-save'
                ]
            ]
        ];
        $this->fields_value['id_voting'] = (int) Tools::getValue('id_voting');
        $voting_option = new VotingOption(Tools::getValue('id_voting_option'));
        $this->tpl_vars = [
            'voting_option' => $voting_option,
        ];
        $this->getlanguages();
        $helper = new HelperForm();
        $helper->show_cancel_button = true;

        $back = Tools::safeOutput(Tools::getValue('back', ''));
        if (empty($back)) {
            $back = self::$currentIndex . '&token=' . $this->token;
        }
        if (!Validate::isCleanHtml($back)) {
            die(Tools::displayError());
        }
        $helper->back_url = $back;
        $helper->currentIndex = self::$currentIndex;
        $helper->token = $this->token;
        $helper->table = $this->table;
        $helper->identifier = $this->identifier;
        $helper->override_folder = 'voting_option/';
        $helper->id = $voting_option->id;
        $helper->toolbar_scroll = false;
        $helper->tpl_vars = $this->tpl_vars;
        $helper->languages = $this->_languages;
        $helper->default_form_language = $this->default_form_language;
        $helper->allow_employee_form_lang = $this->allow_employee_form_lang;
        $helper->fields_value = $this->getFieldsValue($voting_option);
        $helper->toolbar_btn = $this->toolbar_btn;
        $helper->title = 'Add new voting option';
        $this->content .= $helper->generateForm($this->fields_form);
    }

    public function initContent()
    {
        if ($this->display == 'edit' || $this->display == 'add') {
            if (!$this->loadObject(true)) {
                return;
            }
            $this->content .= $this->renderForm();
        } elseif ($this->display == 'view') {
            if ($this->className) {
                $this->loadObject(true);
            }
            $this->content .= $this->renderView();
        } elseif ($this->display == 'editVotingOption') {
            $this->object = new VotingOption((int) Tools::getValue('id_voting_option'));
            $this->content .= $this->initFormVotingOption();
        } elseif ($this->display != 'view' && !$this->ajax) {
            $this->setTypeVoting();
            $this->content .= $this->renderList();
            if (!Tools::getValue('submitFiltervoting_option', 0) && !Tools::getIsset('id_voting_option')) {
                $this->processResetFilters('voting_option');
            }
        }
        $this->context->smarty->assign([
            'content' => $this->content,
        ]);
    }

    public function initProcess()
    {
        if ((Tools::getValue('id_voting_option') && !Tools::getValue('id_voting'))
            || Tools::isSubmit('deletevoting_option')
            || Tools::isSubmit('submitAddvoting_option')
            || Tools::isSubmit('addvoting_option')
            || Tools::isSubmit('updatevoting_option')
            || Tools::isSubmit('submitBulkdeletevoting_option')) {
            $this->setTypeOption();
        }
        if (Tools::getIsset('viewvoting')) {
            $this->list_id = 'voting_option';
            if (isset($_POST['submitReset' . $this->list_id])) {
                $this->processResetFilters();
            }
        } else {
            $this->list_id = 'voting';
        }
        parent::initProcess();
    }

    public function postProcess()
    {
        if (Tools::getIsset('id_voting') && Tools::getIsset('viewvoting')) {
            self::$currentIndex = self::$currentIndex . '&id_voting=' . Tools::getValue('id_voting', 0) . '&viewvoting';
        }
        parent::postProcess();
        if ($this->table == 'voting_option' && ($this->display == 'edit' || $this->display == 'add')) {
            $this->display = 'editVotingOption';
        }
    }

    public function processAdd()
    {
        $object = parent::processAdd();
        if (Tools::isSubmit('submitAdd' . $this->table . 'AndStay') && !count($this->errors)) {
            if ($this->table == 'voting_option' && ($this->display == 'edit' || $this->display == 'add')) {
                $this->redirect_after = self::$currentIndex . '&addvoting_option&id_voting=' . (int) Tools::getValue('id_voting') . '&conf=3&token=' . $this->token;
            } else {
                $this->redirect_after = self::$currentIndex . '&' . $this->identifier . '=&conf=3&update' . $this->table . '&token=' . $this->token;
            }
        } elseif (Tools::isSubmit('submitAdd' . $this->table . 'AndStay') && count($this->errors)) {
            $this->display = 'editVotingOption';
        }
        return $object;
    }

    public function processUpdate()
    {
        $object = parent::processUpdate();
        if (Tools::isSubmit('submitAdd' . $this->table . 'AndStay') && !count($this->errors)) {
            $this->redirect_after = self::$currentIndex . '&' . $this->identifier . '=&conf=3&update' . $this->table . '&token=' . $this->token;
        }
        return $object;
    }

    public function processSave()
    {
        $id_voting = (int) Tools::getValue('id_voting');
        $id_voting_option = (int) Tools::getValue('id_voting_option');
        if ($this->table == 'voting') {
            if ($id_voting <= 0) {
                $sql = 'SELECT `position`+1 FROM `' . _DB_PREFIX_ . 'voting`
                        ORDER BY position DESC';
            }
        } elseif ($this->table == 'voting_option') {
            if ($id_voting_option <= 0) {
                $sql = 'SELECT `position`+1 FROM `' . _DB_PREFIX_ . 'voting_option`
                        WHERE `id_voting` = ' . $id_voting . '
                        ORDER BY position DESC';
            }
        }
        if (isset($sql)) {
            $_POST['position'] = Db::getInstance()->getValue($sql);
        }
        foreach ($_POST as $key => $value) {
            if (preg_match('/^name_/Ui', $key)) {
                $_POST[$key] = str_replace('\n', '', str_replace('\r', '', $value));
            }
        }
        return parent::processSave();
    }

    public function ajaxProcessUpdatePositions()
    {
        if ($this->access('edit')) {
            $way = (int) Tools::getValue('way');
            $id = (int) Tools::getValue('id');
            if (Tools::getIsset('voting_option')) {
                $className = 'VotingOption';
                $identifier = 'voting_option';
                $this->position_identifier = 'id_voting_option';
            } elseif (Tools::getIsset('voting')) {
                $className = 'Voting';
                $identifier = 'voting';
                $this->position_identifier = 'id_voting';
            }
            $positions = Tools::getValue($identifier);
            $new_positions = [];
            foreach ($positions as $v) {
                if (!empty($v)) {
                    $new_positions[] = $v;
                }
            }
            foreach ($new_positions as $position => $value) {
                $pos = explode('_', $value);
                if (isset($pos[2]) && (int) $pos[2] === $id) {
                    if ($object = new $className((int) $pos[2])) {
                        if (isset($position) && $object->updatePosition($way, $position, $id)) {
                            echo 'ok position ' . (int) $position . ' for ' . (int) $pos[1] . '\r\n';
                        } else {
                            echo '{"hasError" : true, "errors" : "Can not update ID' . (int) $id . ' to position ' . (int) $position . ' "}';
                        }
                    } else {
                        echo '{"hasError" : true, "errors" : "This position ID (' . (int) $id . ') can t be loaded"}';
                    }
                    break;
                }
            }
        }
    }

    public function ajaxProcessToggleActiveVotingOption()
    {
        if (!$this->access['edit']) {
            die(json_encode(['success' => 0, 'text' => 'You do not have permission!']));
        }
        $object = new VotingOption((int) Tools::getValue('id_voting_option'));
        $object->active = !$object->active;
        if ($object->save()) {
            die(json_encode(['success' => 1, 'text' => 'Success!']));
        } else {
            die(json_encode(['success' => 0, 'text' => 'Failure!']));
        }
    }

    public function ajaxProcessToggleActiveVoting()
    {
        if (!$this->access['edit']) {
            die(json_encode(['success' => 0, 'text' => 'You do not have permission!']));
        }
        $object = new Voting((int) Tools::getValue('id_voting'));
        $object->active = !$object->active;
        if ($object->save()) {
            die(json_encode(['success' => 1, 'text' => 'Success!']));
        } else {
            die(json_encode(['success' => 0, 'text' => 'Failure!']));
        }
    }

}