<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

require_once(dirname(__FILE__).'/../../classes/Voting.php');
require_once(dirname(__FILE__).'/../../classes/VotingVote.php');

class SimpleVotingsVotesModuleFrontController extends ModuleFrontController
{
    private const SV_COOKIE_POLLING_CYCLE_LIFETIME = 3600; // seconds (3600 - 1 hour)
    private $token;
    private $action;
    private $id_shop;
    private $id_lang;
    private $id_customer;
    private $id_guest;
    private $id_group;
    private $controller_name;
    private $remote_addr;
    private $votings = null;

    public function __construct()
    {
        $this->token = Tools::getToken();
        $this->action = preg_replace('/[^a-z]/', '', Tools::getValue('action'));
        $this->controller_name = preg_replace('/[^a-z]/', '', Tools::getValue('page'));
        if ((Tools::getValue('token') !== $this->token) || !$this->action || !$this->controller_name) {
            $this->dieController();
        }
        if (!in_array($this->action, ['polling', 'vote', 'close', 'dismiss'])) {
            $this->dieController();
        }
        parent::__construct();
        if (!$this->ajax || !in_array($this->controller_name, array_keys($this->module->page_names))) {
            $this->dieController();
        }
    }

    public function init()
    {
        parent::init();

        $this->id_shop = (int) $this->context->shop->id;
        $this->id_lang = (int) $this->context->language->id;
        $this->id_customer = (int) $this->context->customer->id;
        $this->id_guest = (int) $this->context->customer->id_guest;
        $this->id_group = (int) $this->context->customer->id_default_group;
        $this->remote_addr = ip2long(Tools::getRemoteAddr());

        $action = $this->action;
        $cookie = $this->context->cookie;

        if ($action == 'polling') {
            if (!isset($cookie->sv_cookie_exiped)) {
                $cookie->sv_cookie_exiped = time() + self::SV_COOKIE_POLLING_CYCLE_LIFETIME;
            } elseif ($cookie->sv_cookie_exiped < time()) {
                if (isset($cookie->sv_applicable)) {
                    unset($cookie->sv_applicable);
                }
                $cookie->sv_cookie_exiped = time() + self::SV_COOKIE_POLLING_CYCLE_LIFETIME;
            }
            if (!isset($cookie->sv_applicable)) {            
                if ($this->votings = $this->getVotings()) {
                    $cookie->sv_applicable = 1;
                } else {
                    $cookie->sv_applicable = 0;
                    $this->dieController();
                }
            } elseif (!$cookie->sv_applicable) {
                $this->dieController();
            }
            if ($cookie->sv_applicable && !$this->votings) {
                $this->votings = $this->getVotings();
            }
            if (!$this->votings) {
                $cookie->sv_applicable = 0;
                $this->dieController();
            } else {
                $this->displayVotings();
            }
        } elseif ($action == 'vote' || $action == 'close' || $action == 'dismiss') {
            if (!($id_voting = (int) Tools::getValue('id_voting'))) {
                $this->dieController(['applicable' => true]);
            } elseif ($action == 'close' || $action == 'dismiss') {
                $can = false;
                if ($action == 'close' && Voting::canBeClosed($id_voting)) {
                    $can = true;
                }
                if ($action == 'dismiss' && Voting::canBeDismissed($id_voting)) {
                    $can = true;
                    if (!VotingVote::isVoted($id_voting, $this->id_customer, $this->id_guest, $this->remote_addr)) {
                        $vote = new VotingVote();
                        $vote->id_voting = $id_voting;
                        $vote->dismissed = 1;
                        $vote->id_voting_option = 0;
                        $vote->id_customer = $this->id_customer;
                        $vote->id_guest = $this->id_guest;
                        $vote->ip_address = $this->remote_addr;
                        $vote->add();
                    }
                }
                if ($can) {
                    $cookie->sv_applicable = 0;
                    $this->dieController(['applicable' => false]);
                } else {
                    $this->dieController(['applicable' => true]);
                }
            } elseif ($action == 'vote') {
                $options = Tools::getValue('option');
                if (empty($options) || !is_array($options)) {
                    $this->dieController();
                }
                if (!VotingVote::isVoted($id_voting, $this->id_customer, $this->id_guest, $this->remote_addr)) {
                    foreach ($options as $option) {
                        $vote = new VotingVote();
                        $vote->id_voting = $id_voting;
                        $vote->dismissed = 0;
                        $vote->id_voting_option = (int) $option;
                        $vote->id_customer = $this->id_customer;
                        $vote->id_guest = $this->id_guest;
                        $vote->ip_address = $this->remote_addr;
                        $vote->add();
                    }
                    $cookie->sv_applicable = 0;
                    $this->dieController(['applicable' => false, 'thanks' => Voting::getVotingThankYou($id_voting, $this->id_lang)]);
                }
            }
        }
        $this->dieController();
    }

    private function getVotings()
    {
        $id_category = (int) Tools::getValue('id_category');
        $id_product = (int) Tools::getValue('id_product');
        $votings = [];
        $device = 'pc';
        if ((int) $this->context->isMobile()) {
            $device = 'mobile';
        }
        if ((int) $this->context->isTablet()) {
            $device = 'tablet';
        }
        $votings_sql = 'SELECT v.`id_voting`, v.`view`, v.`modal`, v.`multiple`, v.`can_close`, v.`can_dismiss`, v.`show_title`, v.`delay`, v.`position`,
        v.`page_category`, v.`page_product`, vl.`name`, vl.`description`
        FROM `' . _DB_PREFIX_ . 'voting` v
        JOIN `' . _DB_PREFIX_ . 'voting_lang` vl ON vl.`id_voting` = v.`id_voting` AND vl.`id_lang` = ' . (int) $this->id_lang . '
        JOIN `' . _DB_PREFIX_ . 'voting_group` vg ON vg.`id_voting` = v.`id_voting` AND vg.`id_group` = ' . (int) $this->id_group . '
        WHERE v.`active` = 1 AND v.`page_' . $this->controller_name . '` = 1 AND v.`on_' . $device . '` = 1
        GROUP BY v.`id_voting` ORDER BY v.`position` ASC ';
        if ($votings = Db::getInstance()->executeS($votings_sql)) {
            foreach ($votings as $key => $voting) {
                $id_voting = (int) $voting['id_voting'];
                if (VotingVote::isVoted($id_voting, $this->id_customer, $this->id_guest, $this->remote_addr)) {
                    unset($votings[$key]);
                    continue;
                }
                if ($this->controller_name == 'category' || $this->controller_name == 'product') {
                    $obj = new Voting($id_voting);
                    if ($id_category && $categories = $obj->getCategories()) {
                        if (!in_array($id_category, $categories)) {
                            unset($votings[$key]);
                            continue;
                        }
                    }
                    if ($id_product && $products = $obj->getProducts()) {
                        if (!in_array($id_product, $products)) {
                            unset($votings[$key]);
                            continue;
                        }
                    }
                }
                $voting_options_sql = 'SELECT v.`id_voting_option`, v.`id_voting`, v.`show_title`, v.`show_count`,
                vl.`name`, vl.`description`, vl.`button`, COUNT(vv.`id_voting_vote`) `count`
                FROM `' . _DB_PREFIX_ . 'voting_option` v
                JOIN `' . _DB_PREFIX_ . 'voting_option_lang` vl ON vl.`id_voting_option` = v.`id_voting_option` AND vl.`id_lang` = ' . (int) $this->id_lang . '
                LEFT JOIN `' . _DB_PREFIX_ . 'voting_vote` vv ON vv.`id_voting_option` = v.`id_voting_option` AND vv.`dismissed` = 0
                WHERE v.`active` = 1 AND v.`id_voting` = ' . $id_voting . '
                GROUP BY v.`id_voting_option` ORDER BY v.`position` ASC ';
                $voting_options = Db::getInstance()->executeS($voting_options_sql);
                if (!$voting_options) {
                    $this->context->cookie->sv_applicable = 0;
                    $this->dieController(['applicable' => false]);
                } else {
                    $votings[$key]['options'] = $voting_options;
                }
            }
            if (Configuration::get(strtoupper($this->module->name) . '_RSTART')) {
                shuffle($votings);
            }
        }
        return $this->votings = $votings;
    }

    public function displayVotings()
    {
        $response = [];
        if ($this->votings) {
            $response['applicable'] = true;
            foreach ($this->votings as $voting) {
                $this->context->smarty->assign([
                    'voting' => $voting,
                    'sv_link' => $this->context->link->getModuleLink($this->module->name, 'votes')
                ]);
                $tpl = _PS_MODULE_DIR_ . $this->module->name . '/views/templates/front/voting.tpl';
                $response['modal'] = $this->context->smarty->fetch($tpl);
                $response['delay'] = (int) $voting['delay'] * 1000;
                break;
            }
        }
        $this->dieController($response);
    }

    public function dieController($content = [])
    {
        if (empty($content)) {
            $content = [
                'applicable' => false
            ];
        }
        header('Content-Type: application/json; charset=utf-8');
        die($this->ajaxRender(json_encode($content)));
    }
}