<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class SimpleVotings extends Module
{
    public $id_shop_group;
    public $id_shop;
    public $id_lang;

    public $page_names = [
        'index' => 'Home',
        'category' => 'Category',
        'product' => 'Product',
        'cart' => 'Cart',
        'cms' => 'CMS',
        'contact' => 'Contact',
        'order' => 'Order',
        'orderconfirmation' => 'Order Confirmation',
        'manufacturer' => 'Brand',
        'newproducts' => 'New Products',
        'pricesdrop' => 'Prices Drop',
        'bestsales' => 'Best Sales',
        'myaccount' => 'My Account'
    ];

    protected $tabs = [
        [
            'name' => 'Votings',
            'class_name' => 'AdminSimpleVotingsVoting',
            'visible' => true,
            'parent_class_name' => 'AdminParentCustomer',
        ]
    ];

    function __construct()
    {
        $this->name = 'simplevotings';
        $this->version = '1.0.0';
        $this->author = 'metacreo';
        $this->controllers = ['votes'];
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->bootstrap = 1;
        parent::__construct();
        $this->displayName = '1-SimpleVotings';
        $this->description = 'Simple votings, customers survey module for Prestashop';
        $this->ps_versions_compliancy = ['min' => '1.7.7.3', 'max' => _PS_VERSION_];
        $this->id_shop_group = (int) $this->context->shop->id_shop_group;
        $this->id_shop = (int) $this->context->shop->id;
        $this->id_lang = (int) $this->context->language->id;
    }

    public function install()
    {
        Configuration::updateValue(
            strtoupper($this->name) . '_RSTART', 1, false, $this->id_shop_group, $this->id_shop
        );
        Configuration::updateValue(
            strtoupper($this->name) . '_ACTION_MODE', 0, false, $this->id_shop_group, $this->id_shop
        );
        foreach (array_keys($this->page_names) as $v) {
            Configuration::updateValue(
                strtoupper($this->name) . '_' . strtoupper($v), 0, false, $this->id_shop_group, $this->id_shop
            );
        }
        $tables = include dirname(__FILE__) . '/sql/install.php';
        return (parent::install() && $this->manageTables($tables) && $this->registerHook('displayHeader'));
    }

    public function uninstall()
    {
        Configuration::deleteByName(strtoupper($this->name) . '_RSTART');
        Configuration::deleteByName(strtoupper($this->name) . '_ACTION_MODE');
        foreach (array_keys($this->page_names) as $v) {
            Configuration::deleteByName(strtoupper($this->name) . '_' . strtoupper($v));
        }
        $tables = include dirname(__FILE__) . '/sql/uninstall.php';
        return (parent::uninstall() && $this->manageTables($tables) && $this->unregisterHook('displayHeader'));
    }

    private function manageTables($tables)
    {
        foreach ($tables as $query) {
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }
        return true;
    }

    public function hookDisplayHeader($params)
    {
        $php_self = $this->context->controller->php_self ?? null;
        if ($php_self) {
            $php_self_ok = preg_replace('/[^a-z]/', '', $php_self);
            if (!$php_self_ok || !in_array($php_self_ok, array_keys($this->page_names))) {
                return;
            }
            if (!Configuration::get(strtoupper($this->name . '_' . $php_self_ok))) {
                return;
            }
            Media::addJsDef([
                'sv_link' => $this->context->link->getModuleLink($this->name, 'votes'),
                'sv_token' => Tools::getToken(),
                'sv_page' => $php_self_ok,
            ]);
            if ($php_self_ok == 'product') {
                Media::addJsDef(['sv_product' => (int) Tools::getValue('id_product')]);
            } elseif ($php_self_ok == 'category') {
                Media::addJsDef(['sv_category' => (int) Tools::getValue('id_category')]);
            }
            $this->context->controller->registerStylesheet(
                'module-'.$this->name.'-css',
                'modules/'.$this->name.'/views/css/front.css',
                ['media' => 'all', 'priority' => 200, 'version' => 'release-2025-02']
            );

            $this->context->controller->registerJavascript(
                'module-' . $this->name . '-js',
                'modules/' . $this->name . '/views/js/front.js',
                ['priority' => 100, 'attribute' => 'async', 'version' => 'release-2025-02']
            );
        }
    }

    public function getContent()
    {
        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
        if ((Tools::isSubmit('submit' . $this->name)) === true) {
            Configuration::updateValue(
                strtoupper($this->name) . '_RSTART',
                Tools::getValue(strtoupper($this->name) . '_RSTART'),
                false,
                $this->id_shop_group,
                $this->id_shop
            );
            Configuration::updateValue(
                strtoupper($this->name) . '_ACTION_MODE',
                Tools::getValue(strtoupper($this->name) . '_ACTION_MODE'),
                false,
                $this->id_shop_group,
                $this->id_shop
            );
            foreach (array_keys($this->page_names) as $v) {
                Configuration::updateValue(
                    strtoupper($this->name) . '_' . strtoupper($v),
                    Tools::getValue(strtoupper($this->name) . '_' . strtoupper($v)),
                    false,
                    $this->id_shop_group,
                    $this->id_shop
                );
            }
            $this->context->controller->confirmations[] = 'Settings updated';
        }
        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        $inputs = [
            [
                'type' => 'switch',
                'is_bool' => true,
                'label' => 'Randomized startup',
                'desc' => 'Randomized votings startup (On) or startup by order (Off).',
                'name' => strtoupper($this->name) . '_RSTART',
                'values' => [
                    [
                        'id' => 'rand_on',
                        'value' => 1,
                        'label' => 'On'
                    ],
                    [
                        'id' => 'rand_off',
                        'value' => 0,
                        'label' => 'Off'
                    ]
                ]
            ],
            [
                'type' => 'radio',
                'label' => 'Action restriction mode',
                'name' => strtoupper($this->name) . '_ACTION_MODE',
                'values' => [
                    [
                        'id' => 'restrict_0',
                        'value' => 0,
                        'label' => 'Customers and guests detects by ID, no action restriction per IP address.'
                    ],
                    [
                        'id' => 'restrict_1',
                        'value' => 1,
                        'label' => 'Only one action per voting/survey allowed from same IP address for all.'
                    ],
                    [
                        'id' => 'restrict_2',
                        'value' => 2,
                        'label' => 'One action per voting/survey allowed from same IP address for not logged-in customers only.'
                    ]
                ]
            ]
        ];
        foreach ($this->page_names as $k => $v) {
            $inputs[] = [
                'type' => 'switch',
                'is_bool' => true,
                'label' => 'Run module on ' . $v . ' page',
                'name' => strtoupper($this->name) . '_' . strtoupper($k),
                'values' => [
                    [
                        'id' => $k . '_on',
                        'value' => 1,
                        'label' => 'Enable'
                    ],
                    [
                        'id' => $k . '_off',
                        'value' => 0,
                        'label' => 'Disable'
                    ]
                ]
            ];
        }
        $form = [
            'form' => [
                'legend' => [
                    'title' => 'Settings',
                ],
                'input' => $inputs,
                'submit' => [
                    'title' => 'Save settings',
                    'class' => 'btn btn-default pull-right',
                ]
            ]
        ];
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->title = $this->displayName;
        $helper->submit_action = 'submit' . $this->name;
        $helper->fields_value[strtoupper($this->name) . '_RSTART'] = Configuration::get(
            strtoupper($this->name) . '_RSTART',
            $this->id_lang,
            $this->id_shop_group,
            $this->id_shop,
            true
        );
        $helper->fields_value[strtoupper($this->name) . '_ACTION_MODE'] = Configuration::get(
            strtoupper($this->name) . '_ACTION_MODE',
            $this->id_lang,
            $this->id_shop_group,
            $this->id_shop,
            true
        );
        foreach (array_keys($this->page_names) as $v) {
            $helper->fields_value[strtoupper($this->name) . '_' . strtoupper($v)] = Configuration::get(
                strtoupper($this->name) . '_' . strtoupper($v),
                $this->id_lang,
                $this->id_shop_group,
                $this->id_shop,
                true
            );
        }
        return $helper->generateForm([$form]);
    }
}