/* JS */
/* @author metacreo <metacreo@newestvision.com> */

const pollingTimeOut = 3000;

let skipClose = false;
let skipPollingCycle = false;
let stopPolling = false;

let params = {
    ajax: true,
    token: sv_token,
    page: sv_page
};
if (typeof sv_category !== 'undefined') {
    params['id_category'] = sv_category;
}
if (typeof sv_product !== 'undefined') {
    params['id_product'] = sv_product;
}

document.addEventListener('click', function(e) {
    if (!skipClose && (e.target.id == 'svClose')) {
        const svDismiss = document.getElementById('svDismiss');
        const svModal = document.getElementById('svModal');
        params['id_voting'] = svModal.dataset.votingId;
        if (svDismiss && svDismiss.checked) {
            params['action'] = 'dismiss';
        } else {
            params['action'] = 'close';
        }
        let sendData = new FormData();
        Object.entries(params).forEach(([key, value]) => sendData.append(key, value));
        svModal_hide();
        skipPollingCycle = true;
        (async () => {
            const response = await _sendData(sendData);
            if ((typeof response.applicable == 'undefined') || !response.applicable) {
                stopPolling = true;
                return svModal_remove();
            } else {
                skipPollingCycle = false;
                await delay(1000);
                return svModal_show();
            }
        })();
    }
});
document.addEventListener('change', function(e) {
    const checkboxes = Array.from(
        document.querySelectorAll('input[type="checkbox"][name="option[]"]:checked'), e => e.value
    );
    if (checkboxes.length == 0) {
        document.getElementById('svSubmit').disabled = true;
    } else {
        document.getElementById('svSubmit').disabled = false;
    }
});
document.addEventListener('submit', function(e) {
    if (e.target.id == 'svForm') {
        e.preventDefault();
        params['action'] = 'vote';
        const svModal = document.getElementById('svModal');
        params['id_voting'] = svModal.dataset.votingId;
        const form = document.getElementById('svForm');
        let sendData = new FormData(form, e.submitter);
        Object.entries(params).forEach(([key, value]) => sendData.append(key, value));
        skipPollingCycle = true;
        const spiner = '<span class="svBtnLoader"></span>';
        const svSubmit = document.getElementById('svSubmit');
        const svOptBtn = document.querySelectorAll('button.svOptBtn');
        if (svSubmit) {
            svSubmit.disabled = true;
            svSubmit.insertAdjacentHTML('afterBegin', spiner);
        }
        if (svOptBtn) {
            svOptBtn.forEach(button => {
                button.disabled = true;
                if (button.value == e.submitter.value) {
                    button.insertAdjacentHTML('afterBegin', spiner);
                }
            });
        }
        (async () => {
            const response = await _sendData(sendData);
            if (typeof response.thanks !== 'undefined') {
                skipClose = true;
                stopPolling = true;
                await delay(1000);
                const newSvClose = '<button type="button" id="svClose" class="svClose"></button>';
                const svClose = document.getElementById('svClose');
                if (!svClose) {
                    const svModalHeader = document.getElementById('svModalHeader');
                    if (svModalHeader) {
                        svModalHeader.insertAdjacentHTML('beforeEnd', newSvClose);
                    }
                }
                const svModalFooter = document.getElementById('svModalFooter');
                if (svModalFooter) {
                    svModalFooter.remove();
                }
                const svModalBody = document.getElementById('svModalBody');
                if (svModalBody) {
                    const thanksHtml = '<div class="svThanksHeader">' + response.thanks.header +
                          '</div><div class="svThanksText">' + response.thanks.text + '</div>';
                    svModalBody.innerHTML = thanksHtml;
                }
                const svModal = document.getElementById('svModal');
                document.addEventListener('click', function(e) {
                    if (e.target.id == 'svClose') {
                        svModal_hide();
                        return svModal_remove();
                    }
                });
                await delay(5000);
                if (svModal) {
                    svModal_hide();
                    return svModal_remove();
                }
            }
        })();
    }
});
                          
async function sv_polling() {
    if (stopPolling) {
        stopPolling = false;
        return;
    }
    if (skipPollingCycle) {
        await delay(pollingTimeOut);
        return sv_polling();
    }
    params['action'] = 'polling';
    const svModal = document.getElementById('svModal');
    if (svModal && (typeof svModal.dataset.votingId !== 'undefined')) {
        params['id_voting'] = svModal.dataset.votingId;
    }
    let sendData = new FormData();
    Object.entries(params).forEach(([key, value]) => sendData.append(key, value));
    const response = await _sendData(sendData);
    if ((typeof response.applicable == 'undefined') || !response.applicable) {
        return;
    } else if (typeof response.modal !== 'undefined') {
        if (!svModal) {
            svModal_create(response.modal);
            const svModal = document.getElementById('svModal');
        }
        svModal_show();
    }
    await delay(pollingTimeOut);
    sv_polling();
}
sv_polling();

async function _sendData(data) {
    if (!data) {
        return null;
    }
    let response = await fetch(sv_link, {
        method: 'POST',
        body: data
    });
    if (response.status != 200) {
        console.log(response.status);
        await new Promise(resolve => setTimeout(resolve, 5000));
        await _sendData(data);
    } else {
        try {
            return response.json();
        } catch (e) {
            return null;
        }
    }
    return null;
}

async function svModal_create(xHtml) {
    return document.body.insertAdjacentHTML('beforeend', xHtml);
}

async function svModal_remove() {
    const svModal = document.getElementById('svModal');
    if (svModal) {
        return svModal.remove();
    }
}

async function svModal_show(show_delay = 0) {
    if (show_delay > 0) {
        await delay(show_delay);
    }
    const svModal = document.getElementById('svModal');
    if (svModal) {
        return svModal.style.display = 'block';
    }
}

async function svModal_hide() {
    const svModal = document.getElementById('svModal');
    if (svModal) {
        return svModal.style.display = 'none';
    }
}

function delay(time) {
    return new Promise(function(resolve) { 
        setTimeout(resolve, time)
    });
}
